import WebSocket from "ws";
import { User } from "./user.js";
import { Beatmap, Beatmapset } from "./beatmap.js";
import { Multiplayer } from "./multiplayer.js";
import { Score, BeatmapUserScore } from "./score.js";
import { Rankings, Spotlight } from "./ranking.js";
import { Event } from "./event.js";
import { Changelog } from "./changelog.js";
import { Forum, PollConfig } from "./forum.js";
import { WikiPage } from "./wiki.js";
import { News } from "./news.js";
import { SearchResult } from "./home.js";
import { Rulesets, Mod, Scope } from "./misc.js";
import { Chat } from "./chat.js";
export { User } from "./user.js";
export { Beatmap, Beatmapset, RankStatus } from "./beatmap.js";
export { Multiplayer } from "./multiplayer.js";
export { Score, BeatmapUserScore } from "./score.js";
export { Rankings, Spotlight } from "./ranking.js";
export { Event } from "./event.js";
export { Changelog } from "./changelog.js";
export { Forum, PollConfig } from "./forum.js";
export { WikiPage } from "./wiki.js";
export { News } from "./news.js";
export { SearchResult } from "./home.js";
export { Rulesets, Mod, Scope } from "./misc.js";
export { Chat } from "./chat.js";
export { WebSocketEvent } from "./websocket.js";
/**
 * Generates a link for users to click on in order to use your application!
 * @param client_id The Client ID, find it at https://osu.ppy.sh/home/account/edit#new-oauth-application
 * @param redirect_uri The specified Application Callback URL, aka where the user will be redirected upon clicking the button to authorize
 * @param scopes What you want to do with/as the user
 * @param server (defaults to https://osu.ppy.sh) The API server
 * @returns The link people should click on
 */
export declare function generateAuthorizationURL(client_id: number, redirect_uri: string, scopes: Scope[], server?: string): string;
/** If the `API` throws an error, it should always be an `APIError`! */
export declare class APIError {
    message: string;
    server: string;
    endpoint: string;
    parameters?: object;
    /**
     * @param message The reason why things didn't go as expected
     * @param server The server to which the request was sent
     * @param endpoint The type of resource that was requested from the server
     * @param parameters The filters that were used to specify what resource was wanted
     */
    constructor(message: string, server: string, endpoint: string, parameters?: object);
}
/** You can create an API instance using its `createAsync` function! {@link API.createAsync} */
export declare class API {
    client: {
        id: number;
        secret: string;
    };
    /**
     * Should always be "Bearer"
     */
    token_type: string;
    expires: Date;
    access_token: string;
    /**
     * Valid for an unknown amount of time, allows you to get a new token without going through the Authorization Code Grant!
     * Use the API's `refreshToken` function to do that
     */
    refresh_token?: string;
    /**
     * The osu! user id of the user who went through the Authorization Code Grant
     */
    user?: number;
    scopes: Scope[];
    /**
     * (default `none`) Which events should be logged
     */
    verbose: "none" | "errors" | "all";
    /**
     * (default `https://osu.ppy.sh`) The base url of the server where the requests should land
     * @remarks For tokens, requests will be sent to the `oauth/token` route, other requests will be sent to the `api/v2` route
     */
    server: string;
    /**
     * Use the API's `createAsync` instead of the default constructor if you don't have at least an access_token!
     * `createAsync` should always be your way of creating API instances!!
     */
    constructor(client?: {
        id: number;
        secret: string;
    }, token_type?: string, expires?: Date, access_token?: string, scopes?: Scope[], refresh_token?: string, user?: number, verbose?: "none" | "errors" | "all", server?: string);
    /**
     * Use this instead of `console.log` to log any information
     * @param is_error Is the logging happening because of an error?
     * @param to_log Whatever you would put between the parentheses of `console.log()`
     */
    private log;
    /**
     * Set most of an `api`'s properties, like tokens, token_type, scopes, expiration_date
     * @param body An Object with the client id & secret, grant_type, and stuff that depends of the grant_type
     * @param api The `api` which will see its properties change
     * @returns `api`, just in case, because in theory it should modify the original object
     */
    private obtainToken;
    /**
     * The normal way to create an API instance! Make sure to `await` it
     * @param client The ID and the secret of your client, can be found on https://osu.ppy.sh/home/account/edit#new-oauth-application
     * @param user If the instance is supposed to represent a user, use their Authorization Code and the Application Callback URL of your application!
     * @returns A promise with an API instance
     */
    static createAsync(client: {
        id: number;
        secret: string;
    }, user?: {
        code: string;
        redirect_uri: string;
    }, verbose?: "none" | "errors" | "all", server?: string): Promise<API>;
    /** @returns Whether or not the token has been refreshed */
    refreshToken(): Promise<boolean>;
    generateWebSocket(): WebSocket;
    /**
     * @param method The type of request, each endpoint uses a specific one (if it uses multiple, the intent and parameters become different)
     * @param endpoint What comes in the URL after `api/`
     * @param parameters The things to specify in the request, such as the beatmap_id when looking for a beatmap
     * @param number_try Attempt number for doing this specific request
     * @returns A Promise with the API's response
     */
    private request;
    /**
     * Get extensive user data about the authorized user
     * @scope {@link Scope"identify"}
     * @param ruleset Defaults to the user's default/favourite Ruleset
     */
    getResourceOwner(ruleset?: Rulesets): Promise<User.Extended.WithStatisticsrulesets>;
    /**
     * Get extensive user data about whoever you want!
     * @param user An object with either the id or the username of the user you're trying to get
     * @param ruleset Defaults to the user's default/favourite Ruleset
     */
    getUser(user: {
        id?: number;
        username?: string;
    } | User, ruleset?: Rulesets): Promise<User.Extended>;
    /**
     * Get user data for up to 50 users at once!
     * @param users An array of users or of objects that have the id of the users you're trying to get
     */
    getUsers(users: Array<{
        id: number;
    } | User>): Promise<User.WithCountryCoverGroupsStatisticsrulesets[]>;
    /**
     * Get "notable" scores from a user
     * @param user The user who set the scores
     * @param type Do you want scores: in the user's top 100, that are top 1 on a beatmap, that have been recently set?
     * @param limit The maximum amount of scores to be returned
     * @param ruleset The Ruleset the scores were made in, defaults to the user's default/favourite Ruleset
     * @param include_fails (defaults to false) Do you want scores where the user didn't survive or quit the map?
     * @param offset How many elements that would be at the top of the returned array get skipped (while still filling the array up to the limit)
     */
    getUserScores(user: {
        id: number;
    } | User, type: "best" | "firsts" | "recent", limit?: number, ruleset?: Rulesets, include_fails?: boolean, offset?: number): Promise<Score.WithUserBeatmapBeatmapset[]>;
    /**
     * Get beatmaps favourited or made by a user!
     * @param user The user in question
     * @param type The relation between the user and the beatmaps
     * @param limit (defaults to 5) The maximum amount of elements returned in the array
     * @param offset How many elements that would be at the top of the returned array get skipped (while still filling the array up to the limit)
     */
    getUserBeatmaps(user: {
        id: number;
    } | User, type: "favourite" | "graveyard" | "guest" | "loved" | "nominated" | "pending" | "ranked", limit?: number, offset?: number): Promise<Beatmapset.Extended.WithBeatmapExtended[]>;
    /**
     * Get the beatmaps most played by a user!
     * @param user The user who played the beatmaps
     * @param limit (defaults to 5) The maximum amount of elements returned in the array
     * @param offset How many elements that would be at the top of the returned array get skipped (while still filling the array up to the limit)
     */
    getUserMostPlayed(user: {
        id: number;
    } | User, limit?: number, offset?: number): Promise<Beatmap.Playcount[]>;
    /**
     * Get an array of Events of different `type`s that relate to a user's activity during the last 31 days! (or 100 activities, whatever comes first)
     * @param user The user in question
     * @param limit (defaults to 5) The maximum amount of elements returned in the array
     * @param offset How many elements that would be at the top of the returned array get skipped (while still filling the array up to the limit)
     */
    getUserRecentActivity(user: {
        id: number;
    } | User, limit?: number, offset?: number): Promise<Array<Event.AnyRecentActivity>>;
    /**
     * Get data about the activity of a user kudosu-wise!
     * @param user The user in question
     * @param limit (defaults to 5) The maximum amount of activities in the returned array
     * @param offset How many elements that would be at the top of the returned array get skipped (while still filling the array up to the limit)
     */
    getUserKudosu(user: {
        id: number;
    } | User, limit?: number, offset?: number): Promise<User.KudosuHistory[]>;
    /**
     * Get user data of each friend of the authorized user
     * @scope {@link Scope"friends.read"}
     */
    getFriends(): Promise<User.WithCountryCoverGroupsStatisticsSupport[]>;
    /** Get extensive beatmap data about whichever beatmap you want! */
    lookupBeatmap(query: {
        checksum?: string;
        filename?: string;
        id?: number | string;
    }): Promise<Beatmap.Extended.WithFailtimesBeatmapsetextended>;
    /**
     * Get extensive beatmap data about whichever beatmap you want!
     * @param beatmap An object with the id of the beatmap you're trying to get
     */
    getBeatmap(beatmap: {
        id: number;
    } | Beatmap): Promise<Beatmap.Extended.WithFailtimesBeatmapsetextended>;
    /**
     * Get extensive beatmap data for up to 50 beatmaps at once!
     * @param beatmaps An array of beatmaps or of objects that have the id of the beatmaps you're trying to get
     */
    getBeatmaps(beatmaps: Array<{
        id: number;
    } | Beatmap>): Promise<Beatmap.Extended.WithFailtimesMaxcombo[]>;
    /**
     * Get various data about the difficulty of a beatmap!
     * @remarks You may want to use getBeatmapDifficultyAttributesOsu (or Taiko or whatever) instead for better type safety
     * @param beatmap The Beatmap in question
     * @param mods (defaults to No Mod) (will ignore mod settings) Can be a bitset of mods, an array of mod acronyms ("DT" for DoubleTime), or an array of Mods
     * @param ruleset (defaults to the ruleset the beatmap was intended for) Useful to specify if the beatmap is a convert
     */
    getBeatmapDifficultyAttributes(beatmap: {
        id: number;
    } | Beatmap, mods?: Mod[] | string[] | number, ruleset?: Rulesets): Promise<Beatmap.DifficultyAttributes | Beatmap.DifficultyAttributes.Any>;
    /**
     * Get various data about the difficulty of an osu! beatmap!
     * @param beatmap The Beatmap in question
     * @param mods (defaults to No Mod) (will ignore mod settings) Can be a bitset of mods, an array of mod acronyms ("DT" for DoubleTime), or an array of Mods
     */
    getBeatmapDifficultyAttributesOsu(beatmap: {
        id: number;
    } | Beatmap, mods?: Mod[] | string[] | number): Promise<Beatmap.DifficultyAttributes.Osu>;
    /**
     * Get various data about the difficulty of a taiko beatmap!
     * @param beatmap The Beatmap in question
     * @param mods (defaults to No Mod) (will ignore mod settings) Can be a bitset of mods, an array of mod acronyms ("DT" for DoubleTime), or an array of Mods
     */
    getBeatmapDifficultyAttributesTaiko(beatmap: {
        id: number;
    } | Beatmap, mods?: Mod[] | string[] | number): Promise<Beatmap.DifficultyAttributes.Taiko>;
    /**
     * Get various data about the difficulty of a ctb beatmap!
     * @param beatmap The Beatmap in question
     * @param mods (defaults to No Mod) (will ignore mod settings) Can be a bitset of mods, an array of mod acronyms ("DT" for DoubleTime), or an array of Mods
     */
    getBeatmapDifficultyAttributesFruits(beatmap: {
        id: number;
    } | Beatmap, mods?: Mod[] | string[] | number): Promise<Beatmap.DifficultyAttributes.Fruits>;
    /**
     * Get various data about the difficulty of a mania beatmap!
     * @param beatmap The Beatmap in question
     * @param mods (defaults to No Mod) (will ignore mod settings) Can be a bitset of mods, an array of mod acronyms ("DT" for DoubleTime), or an array of Mods
     */
    getBeatmapDifficultyAttributesMania(beatmap: {
        id: number;
    } | Beatmap, mods?: Mod[] | string[] | number): Promise<Beatmap.DifficultyAttributes.Mania>;
    /**
     * Get the top scores of a beatmap!
     * @param beatmap The Beatmap in question
     * @param include_lazer_scores Whether or not lazer scores should be included, defaults to true
     * @param ruleset The Ruleset used to make the scores, useful if they were made on a convert
     * @param mods (2023-11-16) Currently doesn't do anything
     * @param type (2023-11-16) Currently doesn't do anything
     */
    getBeatmapScores(beatmap: {
        id: number;
    } | Beatmap, include_lazer_scores?: boolean, ruleset?: Rulesets, mods?: string[], type?: string): Promise<Score.WithUser[]>;
    /**
     * Get the top scores of a beatmap, in the "solo score" format lazer brought with it!
     * More info on GitHub if needed https://github.com/ppy/osu-infrastructure/blob/master/score-submission.md
     * @param beatmap The Beatmap in question
     * @param ruleset The Ruleset used to make the scores, useful if they were made on a convert
     * @param mods (2023-11-16) Currently doesn't do anything
     * @param type (2023-11-16) Currently doesn't do anything
     */
    getBeatmapSoloScores(beatmap: {
        id: number;
    } | Beatmap, ruleset?: Rulesets, mods?: string[], type?: string): Promise<Score.Solo[]>;
    /**
     * Get the score on a beatmap made by a specific user (with specific mods and on a specific ruleset if needed)
     * @param beatmap The Beatmap the score was made on
     * @param user The User who made the score
     * @param mods The Mods used to make the score, defaults to any, you can use `["NM"]` to filter out scores with mods
     * @param ruleset The Ruleset used to make the score, useful if it was made on a convert
     * @returns An Object with the position of the score according to the specified Mods and Ruleset, and with the score itself
     */
    getBeatmapUserScore(beatmap: {
        id: number;
    } | Beatmap, user: {
        id: number;
    } | User, mods?: string[], ruleset?: Rulesets): Promise<BeatmapUserScore>;
    /**
     * Get the score on a beatmap made by a specific user (with the possibility to specify if the scores are on a convert)
     * @param beatmap The Beatmap the scores were made on
     * @param user The User who made the scores
     * @param ruleset The Ruleset used to make the scores, defaults to the Ruleset the Beatmap was made for
     */
    getBeatmapUserScores(beatmap: {
        id: number;
    } | Beatmap, user: {
        id: number;
    } | User, ruleset?: Rulesets): Promise<Score.Legacy[]>;
    /**
     * Get extensive beatmapset data about whichever beatmapset you want!
     * @param beatmapset An object with the id of the beatmapset you're trying to get
     */
    getBeatmapset(beatmapset: {
        id: number;
    } | Beatmapset): Promise<Beatmapset.Extended.Plus>;
    /**
     * Get data about a BeatmapPack using its tag!
     * @param pack An object with the tag of the beatmappack you're trying to get
     * @remarks Currently in https://osu.ppy.sh/beatmaps/packs, when hovering a pack, its link with its tag should show up on your browser's bottom left
     */
    getBeatmapPack(pack: {
        tag: string;
    } | Beatmap.Pack): Promise<Beatmap.Pack>;
    /**
     * Get an Array of up to 100 BeatmapPacks of a specific type!
     * @param type The type of the BeatmapPacks, defaults to "standard"
     */
    getBeatmapPacks(type?: "standard" | "featured" | "tournament" | "loved" | "chart" | "theme" | "artist"): Promise<Beatmap.Pack[]>;
    /**
     * Get details about the version/update/build of something related to osu!
     * @param changelog A build version like `2023.1026.0`, a stream name like `lazer` or the id of a build
     * @param is_id Whether or not `changelog` is the id of a build, defaults to false
     * @param message_formats Elements of `changelog_entries` will have a `message` property if `markdown`, `message_html` property if `html`, defaults to both
     */
    lookupChangelogBuild(changelog: string, is_id?: boolean, message_formats?: ("html" | "markdown")[]): Promise<Changelog.Build.WithChangelogentriesVersions>;
    /**
     * Get details about the version/update/build of something related to osu!
     * @param stream The name of the thing related to osu!, like `lazer`, `web`, `cuttingedge`, `beta40`, `stable40`
     * @param build The name of the version! Usually something like `2023.1026.0` for lazer, or `20230326` for stable
     */
    getChangelogBuild(stream: string, build: string): Promise<Changelog.Build.WithChangelogentriesVersions>;
    /**
     * Get up to 21 versions/updates/builds!
     * @param versions Get builds that were released before/after (and including) those versions (use the name of the versions, e.g. `2023.1109.0`)
     * @param max_id Filter out builds that have an id higher than this (this takes priority over `versions.to`)
     * @param stream Only get builds from a specific stream
     * @param message_formats Elements of `changelog_entries` will have a `message` property if `markdown`, `message_html` property if `html`, defaults to both
     */
    getChangelogBuilds(versions?: {
        from?: string;
        to?: string;
    }, max_id?: number, stream?: string, message_formats?: ("html" | "markdown")[]): Promise<Changelog.Build.WithUpdatestreamsChangelogentries[]>;
    /**
     * An effective way to get all available streams, as well as their latest version!
     * @example
     * ```ts
     * const names_of_streams = (await api.getChangelogStreams()).map(s => s.name)
     * ```
     */
    getChangelogStreams(): Promise<Changelog.UpdateStream.WithLatestbuildUsercount[]>;
    /**
     * Get data about a lazer multiplayer room (realtime or playlists)!
     * @param room An object with the id of the room, is at the end of its URL (after `/multiplayer/rooms/`)
     */
    getRoom(room: {
        id: number;
    } | Multiplayer.Room): Promise<Multiplayer.Room>;
    /**
     * Get playlists/realtime rooms that are active, that have ended, that the user participated in, that the user made, or just simply any room!
     * @scope {@link Scope"public"}
     * @param type Whether the multiplayer rooms are in playlist format (like current spotlights) or realtime
     * @param mode The state of the room, or the relation of the authorized user with the room
     * @param limit The maximum amount of rooms to return, defaults to 10
     * @param sort Sort (where most recent is first) by creation date or end date, defaults to the creation date
     */
    getRooms(type: "playlists" | "realtime", mode: "active" | "all" | "ended" | "participated" | "owned", limit?: number, sort?: "ended" | "created"): Promise<Multiplayer.Room[]>;
    /**
     * Get the room stats of all the users of that room!
     * @scope {@link Scope"public"}
     * @param room An object with the id of the room in question
     */
    getRoomLeaderboard(room: {
        id: number;
    } | Multiplayer.Room): Promise<Multiplayer.Leader[]>;
    /**
     * Get the scores on a specific item of a room!
     * @param item An object with the id of the item in question, as well as the id of the room
     * @param limit How many scores maximum? Defaults to 50, the maximum the API will return
     * @param sort Sort by scores, ascending or descending? Defaults to descending
     * @param cursor_string Use a MultiplayerScores' `params` and `cursor_string` to get the next page (scores 51 to 100 for example)
     * @remarks (2024-03-04) This may not work for rooms from before March 5th, use at your own risk
     * https://github.com/ppy/osu-web/issues/10725
     */
    getPlaylistItemScores(item: {
        id: number;
        room_id: number;
    } | Multiplayer.PlaylistItem, limit?: number, sort?: "score_asc" | "score_desc", cursor_string?: string): Promise<Multiplayer.Scores>;
    /**
     * Get data of a multiplayer lobby from the stable (non-lazer) client that have URLs with `community/matches` or `mp`
     * @param id Can be found at the end of the URL of said match
     */
    getMatch(id: number): Promise<Multiplayer.Match>;
    /**
     * Gets the info of the 50 most recently created stable (non-lazer) matches, descending order (most recent is at index 0)
     */
    getMatches(): Promise<Multiplayer.MatchInfo[]>;
    /**
     * Get the top 50 players who have the most total kudosu!
     */
    getKudosuRanking(): Promise<User.WithKudosu[]>;
    /**
     * Get the top players of the game, with some filters!
     * @param ruleset Self-explanatory, is also known as "Gamemode"
     * @param type Rank players by their performance points or by their ranked score?
     * @param page (defaults to 1) Imagine `Rankings` as a page, it can only have a maximum of 50 players, while 50 others may be on the next one
     * @param filter What kind of players do you want to see? Defaults to `all`, `friends` has no effect if no authorized user
     * @param country Only get players from a specific country, using its ISO 3166-1 alpha-2 country code! (France would be `FR`, United States `US`)
     * @param variant If `type` is `performance` and `ruleset` is mania, choose between 4k and 7k!
     */
    getUserRanking(ruleset: Rulesets, type: "performance" | "score", page?: number, filter?: "all" | "friends", country?: string, variant?: "4k" | "7k"): Promise<Rankings.User>;
    /**
     * Get the top countries of a specific ruleset!
     * @param ruleset On which Ruleset should the countries be compared?
     * @param page (defaults to 1) Imagine `Rankings` as a page, it can only have a maximum of 50 countries, while 50 others may be on the next one
     */
    getCountryRanking(ruleset: Rulesets, page?: number): Promise<Rankings.Country>;
    /**
     * Get the rankings of a spotlight from 2009 to 2020 on a specific ruleset!
     * @param ruleset Each spotlight has a different ranking (and often maps) depending on the ruleset
     * @param spotlight The spotlight in question
     * @param filter What kind of players do you want to see? Defaults to `all`, `friends` has no effect if no authorized user
     */
    getSpotlightRanking(ruleset: Rulesets, spotlight: {
        id: number;
    } | Spotlight, filter?: "all" | "friends"): Promise<Rankings.Spotlight>;
    /**
     * Get ALL legacy spotlights! (2009-2020, somewhat known as charts/ranking charts, available @ https://osu.ppy.sh/rankings/osu/charts)
     * @remarks The data for newer spotlights (2020-, somewhat known as seasons) can be obtained through `getRoom()`
     * but you can't really get their id without going through the website's URLs (https://osu.ppy.sh/seasons/latest) as far as I know :(
     */
    getSpotlights(): Promise<Spotlight[]>;
    /**
     * Look for a user like you would on the website!
     * @param query What you would put in the searchbar
     * @param page (defaults to 1) You normally get the first 20 results, but if page is 2, you'd get results 21 to 40 instead for example!
     */
    searchUser(query: string, page?: number): Promise<SearchResult.User>;
    /**
     * Look for a wiki page like you would on the website!
     * @param query What you would put in the searchbar
     * @param page (defaults to 1) You normally get the first 50 results, but if page is 2, you'd get results 51 to 100 instead for example!
     */
    searchWiki(query: string, page?: number): Promise<SearchResult.Wiki>;
    /**
     * Get a wiki page!
     * @param path What's in the page's URL after `https://osu.ppy.sh/wiki/` (so the title, after the subtitle if there is a subtitle)
     * (An example for `https://osu.ppy.sh/wiki/en/Game_mode/osu!` would be `Game_mode/osu!`)
     * @param locale (defaults to "en") The BCP 47 language (sub)tag, lowercase (for example, for the article in french, use "fr")
     */
    getWikiPage(path: string, locale?: string): Promise<WikiPage>;
    /**
     * Get all the NewsPosts of a specific year!
     * @remarks If the specified year is invalid/has no news, it fallbacks to the default year
     * @param year (defaults to current year) The year the posts were made
     */
    getNewsPosts(year?: number): Promise<News.Post[]>;
    /**
     * Get a NewsPost, its content, and the NewsPosts right before and right after it!
     * @param post An object with the id or the slug of a NewsPost (the slug being the filename minus the extension, used in its URL)
     */
    getNewsPost(post: {
        id?: number;
        slug?: string;
    } | News.Post): Promise<News.PostWithContentNavigation>;
    /**
     * Make and send a ForumPost in a ForumTopic!
     * @scope {@link Scope"forum.write"}
     * @param topic An object with the id of the topic you're making your reply in
     * @param text Your reply! Your message!
     * @returns The reply you've made!
     */
    replyForumTopic(topic: {
        id: number;
    } | Forum.Topic, text: string): Promise<Forum.Post>;
    /**
     * Create a new ForumTopic in the forum of your choice!
     * @scope {@link Scope"forum.write"}
     * @remarks Some users may not be allowed to do that, such as newly registered users, so this can 403 even with the right scopes
     * @param forum_id The id of the forum you're creating your topic in
     * @param title The topic's title
     * @param text The first post's content/message
     * @param poll If you want to make a poll, specify the parameters of that poll!
     * @returns An object with the topic you've made, and its first initial post (which uses your `text`)
     */
    createForumTopic(forum_id: number, title: string, text: string, poll?: PollConfig): Promise<{
        topic: Forum.Topic;
        post: Forum.Post;
    }>;
    /**
     * Get a forum topic, as well as its main post (content) and the posts that were sent in it!
     * @remarks The oldest post of a topic is the text of a topic
     * @param topic An object with the id of the topic in question
     * @param limit (defaults to 20, max 50) How many `posts` maximum?
     * @param sort (defaults to "id_asc") "id_asc" to have the oldest post at the beginning of the `posts` array, "id_desc" to have the newest instead
     * @param first_post (ignored if `cursor_string`) An Object with the id of the first post to be returned in `posts`
     * @param cursor_string Use a response's `cursor_string` with the same parameters to get the next "page" of results, so `posts` in this instance!
     */
    getForumTopicAndPosts(topic: {
        id: number;
    } | Forum.Topic, limit?: number, sort?: "id_asc" | "id_desc", first_post?: {
        id: number;
    } | Forum.Post, cursor_string?: string): Promise<{
        posts: Forum.Post[];
        topic: Forum.Topic;
        cursor_string: string;
    }>;
    /**
     * Edit the title of a ForumTopic!
     * @scope {@link Scope"forum.write"}
     * @remarks Use `editForumPost` if you wanna edit the post at the top of the topic
     * @param topic An object with the id of the topic in question
     * @param new_title The new title of the topic
     * @returns The edited ForumTopic
     */
    editForumTopicTitle(topic: {
        id: number;
    } | Forum.Topic, new_title: string): Promise<Forum.Topic>;
    /**
     * Edit a ForumPost! Note that it can be the initial one of a ForumTopic!
     * @scope {@link Scope"forum.write"}
     * @param post An object with the id of the post in question
     * @param new_text The new content of the post (replaces the old content)
     * @returns The edited ForumPost
     */
    editForumPost(post: {
        id: number;
    } | Forum.Post, new_text: string): Promise<Forum.Post>;
    /**
     * Needs to be done periodically to reset chat activity timeout
     * @scope {@link Scope"chat.read"}
     * @remarks Every 30 seconds is a good idea
     * @param since UserSilences that are before that will not be returned!
     * @returns A list of recent silences
     */
    keepChatAlive(since?: {
        user_silence?: {
            id: number;
        } | Chat.UserSilence;
        message?: {
            message_id: number;
        } | Chat.Message;
    }): Promise<Chat.UserSilence[]>;
    /**
     * Send a private message to someone!
     * @scope {@link Scope"chat.write"}
     * @remarks You don't need to use `createChatPrivateChannel` before sending a message
     * @param user_target The User you wanna send your message to!
     * @param message The message you wanna send
     * @param is_action (defaults to false) Is it a command? Like `/me dances`
     * @param uuid A client-side message identifier
     * @returns The message you sent
     */
    sendChatPrivateMessage(user_target: {
        id: number;
    } | User, message: string, is_action?: boolean, uuid?: string): Promise<{
        channel: Chat.Channel;
        message: Chat.Message;
    }>;
    /**
     * Get the recent messages of a specific ChatChannel!
     * @scope {@link Scope"chat.read"}
     * @param channel The Channel you wanna get the messages from
     * @param limit (defaults to 20, max 50) The maximum amount of messages you want to get!
     * @param since Get the messages sent after this message
     * @param until Get the messages sent up to but not including this message
     */
    getChatMessages(channel: {
        channel_id: number;
    } | Chat.Channel, limit?: number, since?: {
        message_id: number;
    } | Chat.Message, until?: {
        message_id: number;
    } | Chat.Message): Promise<Chat.Message[]>;
    /**
     * Send a message in a ChatChannel!
     * @scope {@link Scope"chat.write"}
     * @param channel The channel in which you want to send your message
     * @param message The message you wanna send
     * @param is_action (defaults to false) Is it a command? Like `/me dances`
     * @returns The newly sent ChatMessage!
     */
    sendChatMessage(channel: {
        channel_id: number;
    } | Chat.Channel, message: string, is_action?: boolean): Promise<Chat.Message>;
    /**
     * Join a public or multiplayer ChatChannel, allowing you to interact with it!
     * @scope {@link Scope"chat.write_manage"}
     * @param channel The channel you wanna join
     * @param user (defaults to the presumed authorized user) The user joining the channel
     */
    joinChatChannel(channel: {
        channel_id: number;
    } | Chat.Channel, user?: {
        id: number;
    } | User): Promise<Chat.Channel.WithDetails>;
    /**
     * Leave/Close a public ChatChannel!
     * @scope {@link Scope"chat.write_manage"}
     * @param channel The channel you wanna join
     * @param user (defaults to the presumed authorized user) The user joining the channel
     */
    leaveChatChannel(channel: {
        channel_id: number;
    } | Chat.Channel, user?: {
        id: number;
    } | User): Promise<void>;
    /**
     * Mark a certain channel as read up to a given message!
     * @scope {@link Scope"chat.read"}
     * @param channel The channel in question
     * @param message You're marking this and all the messages before it as read!
     */
    markChatChannelAsRead(channel: {
        channel_id: number;
    } | Chat.Channel, message: {
        message_id: number;
    } | Chat.Message): Promise<void>;
    /**
     * Get a list of all publicly joinable channels!
     * @scope {@link Scope"chat.read"}
     */
    getChatChannels(): Promise<Chat.Channel[]>;
    /**
     * Create/Open/Join a private messages chat channel!
     * @scope {@link Scope"chat.read"}
     * @param user_target The other user able to read and send messages in this channel
     * @returns The newly created channel!
     */
    createChatPrivateChannel(user_target: {
        id: number;
    } | User): Promise<Chat.Channel>;
    /**
     * Create a new announcement!
     * @scope {@link Scope"chat.write_manage"}
     * @remarks From my understanding, this WILL 403 unless the user is kinda special
     * @param channel Details of the channel you're creating
     * @param user_targets The people that will receive your message
     * @param message The message to send with the announcement
     * @returns The newly created channel!
     */
    createChatAnnouncementChannel(channel: {
        name: string;
        description: string;
    }, user_targets: Array<{
        id: number;
    } | User>, message: string): Promise<Chat.Channel>;
    /**
     * Get a ChatChannel, and the users in it if it is a private channel!
     * @scope {@link Scope"chat.read"}
     * @remarks Will 404 if the user has not joined the channel (use `joinChatChannel` for that)
     * @param channel The channel in question
     */
    getChatChannel(channel: {
        channel_id: number;
    } | Chat.Channel): Promise<Chat.Channel.WithDetails>;
    /**
     * Get the replay for a score!
     * @scope {@link Scope"public"}
     * @param score The score that has created the replay
     * @returns The correctly encoded content of what would be a replay file (you can just fs.writeFileSync with it!)
     */
    getReplay(score: {
        id: number;
    } | Score): Promise<string>;
}
